/* 
   This file is part of libodbc++.
   
   Copyright (C) 1999-2000 Manush Dodunekov <manush@stendahls.net>
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "mainwindow.h"
#include "connectwindow.h"
#include "resultwindow.h"

#include <qlayout.h>
#include <qsplitter.h>
#include <qstatusbar.h>
#include <qmessagebox.h>

#include <odbc++/drivermanager.h>
#include <odbc++/connection.h>
#include <odbc++/statement.h>
#include <odbc++/resultset.h>
#include <odbc++/resultsetmetadata.h>

#include <memory>

using namespace odbc;

MainWindow::MainWindow()
  :QMainWindow(NULL,NULL),
   con_(NULL)
{
  this->setCaption("libodbc++_qt test app");
  fileMenu_=new QPopupMenu(this);

  connectId_=fileMenu_->insertItem("&Connect",this,SLOT(connect()));

  disconnectId_=fileMenu_->insertItem("&Disconnect",this,SLOT(disconnect()));
  fileMenu_->setItemEnabled(disconnectId_,false);

  fileMenu_->insertSeparator();

  fileMenu_->insertItem("E&xit",this,SLOT(fileExit()));

  menuBar()->insertItem("&File",fileMenu_);

  QWidget* c=new QWidget(this);
  this->setCentralWidget(c);
  QBoxLayout* vl=new QVBoxLayout(c);

  QBoxLayout* hl=new QHBoxLayout();

  sqlEdit_=new QMultiLineEdit(c);
  vl->addWidget(new QLabel(sqlEdit_,"&SQL:",c));
  vl->addWidget(sqlEdit_);
  vl->addLayout(hl);

  QPushButton* exec=new QPushButton
    ("&Execute",c);
  hl->addWidget(exec);

  QObject::connect(exec,SIGNAL(clicked()),this,SLOT(executeQuery()));

  autoCommit_=new QPushButton("&Autocommit",c);
  autoCommit_->setToggleButton(true);
  autoCommit_->setOn(false);
  hl->addWidget(autoCommit_);
  
  QObject::connect(autoCommit_,SIGNAL(clicked()),this,SLOT(autoCommit()));

  QPushButton* b=new QPushButton("&Commit",c);
  hl->addWidget(b);
  QObject::connect(b,SIGNAL(clicked()),this,SLOT(commit()));

  b=new QPushButton("&Rollback",c);
  hl->addWidget(b);
  QObject::connect(b,SIGNAL(clicked()),this,SLOT(rollback()));

  // activate status bar
  (void)this->statusBar();
}


MainWindow::~MainWindow()
{
}

void MainWindow::fileExit()
{
  this->disconnect();
  this->close();
}

void MainWindow::executeQuery()
{
  if(this->checkConnected()) {
    int l=sqlEdit_->numLines();
    QString sql;
    
    for(int i=0; i<l; i++) {
      if(i>0) {
	sql+="\n";
      }
      sql+=sqlEdit_->textLine(i);
    }
    
    try {
      std::auto_ptr<Statement> stmt(con_->createStatement());

      if(stmt->execute(sql)) {
	ResultWindow* rw=new ResultWindow(stmt.release());
      } else {
	statusBar()->message
	  ("Query executed, "+
	   QString::number(stmt->getUpdateCount())+
	   " rows affected",5000);
      }

    } catch(exception& e) {
      QMessageBox::warning
	(this,"Query failed",e.what(),
	 QMessageBox::Ok,0,0);
    }
  }
}

void MainWindow::connect()
{
  auto_ptr<ConnectWindow> cw(new ConnectWindow(this));

  if(cw->exec()==QDialog::Accepted) {
    fileMenu_->setItemEnabled(connectId_,false);
    fileMenu_->setItemEnabled(disconnectId_,true);
    con_=cw->getConnection();

    try {
      autoCommit_->setOn(con_->getAutoCommit());
      autoCommit_->setEnabled(true);
    } catch(SQLException& e) {
      autoCommit_->setEnabled(false);
    }

    this->statusBar()->message("Connected",3000);
  }
}

void MainWindow::disconnect()
{
  if(con_!=NULL) {
    delete con_;
    con_=NULL;
    statusBar()->message("Disconnected",3000);
    fileMenu_->setItemEnabled(disconnectId_,false);
    fileMenu_->setItemEnabled(connectId_,true);
  }
}

bool MainWindow::checkConnected()
{
  if(con_==NULL) {
    QMessageBox::warning
      (this,"Not yet",
       "Please connect to a data source first",QMessageBox::Ok,0,0);
    return false;
  }
  return true;
}

void MainWindow::autoCommit()
{
  if(this->checkConnected()) {
    bool b=autoCommit_->isOn();
    try {
      con_->setAutoCommit(b);
      autoCommit_->setOn(b);
      statusBar()->message(QString("Autocommit is now ")+(b?"on":"off"),3000);
    } catch(SQLException& e) {
      QMessageBox::warning
	(this,"Whoops",
	 e.getMessage(),QMessageBox::Ok,0,0);
      autoCommit_->setOn(!b);
    }
  }
}

void MainWindow::commit()
{
  if(this->checkConnected()) {
    try {
      con_->commit();
      statusBar()->message("Transaction committed",3000);
    } catch(SQLException& e) {
      QMessageBox::warning
	(this,"Commit failed",
	 e.getMessage(),QMessageBox::Ok,0,0);
    }
  }
}

void MainWindow::rollback()
{
  if(this->checkConnected()) {
    try {
      con_->rollback();
      statusBar()->message("Transaction rolled back",3000);
    } catch(SQLException& e) {
      QMessageBox::warning
	(this,"Rollback failed",
	 e.getMessage(),QMessageBox::Ok,0,0);
    }
  }
}
